@@ -246,12 +246,12 @@ const char *canonical;  /* server name */
     if (retval) {
 #ifdef HEIMDAL
       if (err_ret && err_ret->e_text) {
-          report(stderr, GT_("krb5_sendauth: %s [server says '%*s'] \n"),
+          report(stderr, GT_("krb5_sendauth: %s [server says '%s']\n"),
                  error_message(retval),
                  err_ret->e_text);
 #else
       if (err_ret && err_ret->text.length) {
-          report(stderr, GT_("krb5_sendauth: %s [server says '%*s'] \n"),
+          report(stderr, GT_("krb5_sendauth: %s [server says '%*s']\n"),
 		 error_message(retval),
 		 err_ret->text.length,
 		 err_ret->text.data);
@@ -492,10 +492,13 @@ static int fetch_messages(int mailserver_socket, struct query *ctl,
 
 	/* check if the message is old
 	 * Note: the size of the message may not be known here */
-	if (ctl->fetchall || force_retrieval)
-	    ;
-	else if (ctl->server.base_protocol->is_old && (ctl->server.base_protocol->is_old)(mailserver_socket,ctl,num))
-	    msgcode = MSGLEN_OLD;
+	if (ctl->fetchall || force_retrieval) {
+	    /* empty */
+	} else { 
+	    if (ctl->server.base_protocol->is_old && (ctl->server.base_protocol->is_old)(mailserver_socket,ctl,num)) {
+		    msgcode = MSGLEN_OLD;
+	    }
+        }
 	if (msgcode == MSGLEN_OLD)
 	{
   		/* To avoid flooding the syslog when using --keep,
@@ -1518,7 +1521,6 @@ is restored."));
 	smtp_close(ctl, 0);
 	if (mailserver_socket != -1) {
 	    cleanupSockClose(mailserver_socket);
-	    mailserver_socket = -1;
 	}
 	/* If there was a connect timeout, the socket should be closed.
 	 * mailserver_socket_temp contains the socket to close.
