@@ -72,9 +72,9 @@ flag versioninfo;	    /* emit only version info */
 char *user;		    /* the name of the invoking user */
 char *home;		    /* invoking user's home directory */
 char *fmhome;		    /* fetchmail's home directory */
-char *program_name;	    /* the name to prefix error messages with */
+const char *program_name;   /* the name to prefix error messages with */
 flag configdump;	    /* dump control blocks for configurator */
-char *fetchmailhost;	    /* either `localhost' or the host's FQDN */
+const char *fetchmailhost;  /* either `localhost' or the host's FQDN */
 
 static int quitonly;	    /* if we should quit after killing the running daemon */
 
@@ -135,9 +135,10 @@ static RETSIGTYPE donothing(int sig)
 
 static void printcopyright(FILE *fp) {
 	fprintf(fp, GT_("Copyright (C) 2002, 2003 Eric S. Raymond\n"
-		   "Copyright (C) 2004 Matthias Andree, Eric S. Raymond, Robert M. Funk, Graham Wilson\n"
-		   "Copyright (C) 2005 - 2006 Sunil Shetye\n"
-		   "Copyright (C) 2005 - 2009 Matthias Andree\n"
+		   "Copyright (C) 2004 Matthias Andree, Eric S. Raymond,\n"
+		   "                   Robert M. Funk, Graham Wilson\n"
+		   "Copyright (C) 2005 - 2006, 2010 Sunil Shetye\n"
+		   "Copyright (C) 2005 - 2010 Matthias Andree\n"
 		   ));
 	fprintf(fp, GT_("Fetchmail comes with ABSOLUTELY NO WARRANTY. This is free software, and you\n"
 		   "are welcome to redistribute it under certain conditions. For details,\n"
@@ -583,7 +584,7 @@ int main(int argc, char **argv)
 		exit(rc);
 	    }
 	}
-	report(stdout, GT_("starting fetchmail %s daemon \n"), VERSION);
+	report(stdout, GT_("starting fetchmail %s daemon\n"), VERSION);
 
 	/*
 	 * We'll set up a handler for these when we're sleeping,
@@ -599,9 +600,9 @@ int main(int argc, char **argv)
 	if (run.logfile && !nodetach && access(run.logfile, F_OK) == 0)
     	{
 	    if (!freopen(run.logfile, "a", stdout))
-		    report(stderr, GT_("could not open %s to append logs to \n"), run.logfile);
+		    report(stderr, GT_("could not open %s to append logs to\n"), run.logfile);
 	    if (!freopen(run.logfile, "a", stderr))
-		    report(stdout, GT_("could not open %s to append logs to \n"), run.logfile);
+		    report(stdout, GT_("could not open %s to append logs to\n"), run.logfile);
 	    if (run.use_syslog)
 		report(stdout, GT_("fetchmail: Warning: syslog and logfile are set. Check both for logs!\n"));
     	}
@@ -943,6 +944,7 @@ static void optmerge(struct query *h2, struct query *h1, int force)
     FLAG_MERGE(server.plugin);
     FLAG_MERGE(server.plugout);
     FLAG_MERGE(server.tracepolls);
+    FLAG_MERGE(server.badheader);
 
     FLAG_MERGE(wildcard);
     FLAG_MERGE(remotename);
@@ -1011,6 +1013,7 @@ static int load_params(int argc, char **argv, int optind)
     def_opts.server.protocol = P_AUTO;
     def_opts.server.timeout = CLIENT_TIMEOUT;
     def_opts.server.esmtp_name = user;
+    def_opts.server.badheader = BHREJECT;
     def_opts.warnings = WARNING_INTERVAL;
     def_opts.remotename = user;
     def_opts.listener = SMTP_MODE;
@@ -1029,11 +1032,14 @@ static int load_params(int argc, char **argv, int optind)
     }
 
     /* note the parse time, so we can pick up on modifications */
-    parsetime = 0;	/* foil compiler warnings */
-    if (strcmp(rcfile, "-") == 0 || stat(rcfile, &rcstat) != -1)
-	parsetime = rcstat.st_mtime;
-    else if (errno != ENOENT)
-	report(stderr, GT_("couldn't time-check the run-control file\n"));
+    if (strcmp(rcfile, "-") == 0)
+	parsetime = time(NULL);
+    else {
+	if (stat(rcfile, &rcstat) != -1)
+	    parsetime = rcstat.st_mtime;
+	else if (errno != ENOENT)
+	    report(stderr, GT_("couldn't time-check the run-control file\n"));
+    }
 
     /* this builds the host list */
     if ((st = prc_parse_file(rcfile, !versioninfo)) != 0)
@@ -1118,17 +1124,17 @@ static int load_params(int argc, char **argv, int optind)
     if (cmd_run.poll_interval >= 0)
 	run.poll_interval = cmd_run.poll_interval;
     if (cmd_run.invisible)
-	run.invisible = cmd_run.invisible;
+	run.invisible = (cmd_run.invisible == FLAG_TRUE);
     if (cmd_run.showdots)
-	run.showdots = cmd_run.showdots;
+	run.showdots = (cmd_run.showdots == FLAG_TRUE);
     if (cmd_run.use_syslog)
 	run.use_syslog = (cmd_run.use_syslog == FLAG_TRUE);
     if (cmd_run.postmaster)
 	run.postmaster = cmd_run.postmaster;
     if (cmd_run.bouncemail)
-	run.bouncemail = cmd_run.bouncemail;
+	run.bouncemail = (cmd_run.bouncemail == FLAG_TRUE);
     if (cmd_run.softbounce)
-	run.softbounce = cmd_run.softbounce;
+	run.softbounce = (cmd_run.softbounce == FLAG_TRUE);
 
     /* check and daemon options are not compatible */
     if (check_only && run.poll_interval)
@@ -1554,7 +1560,7 @@ static void dump_params (struct runctl *runp,
 	printf(GT_("Fetchmail will direct error mail to the sender.\n"));
 
     if (!runp->softbounce)
-	printf(GT_("Fetchmail will treat permanent errors as permanent (drop messsages).\n"));
+	printf(GT_("Fetchmail will treat permanent errors as permanent (drop messages).\n"));
     else if (outlevel >= O_VERBOSE)
 	printf(GT_("Fetchmail will treat permanent errors as temporary (keep messages).\n"));
 
@@ -1946,7 +1952,17 @@ static void dump_params (struct runctl *runp,
         if (ctl->server.tracepolls)
             printf(GT_("  Poll trace information will be added to the Received header.\n"));
         else if (outlevel >= O_VERBOSE)
-            printf(GT_("  No poll trace information will be added to the Received header.\n.\n"));
+            printf(GT_("  No poll trace information will be added to the Received header.\n"));
+
+	switch (ctl->server.badheader) {
+	    case BHREJECT:
+		if (outlevel >= O_VERBOSE)
+		    printf(GT_("  Messages with bad headers will be rejected.\n"));
+		break;
+	    case BHACCEPT:
+		printf(GT_("  Messages with bad headers will be passed on.\n"));
+		break;
+	}
 
 	if (ctl->properties)
 	    printf(GT_("  Pass-through properties \"%s\".\n"),
