@@ -145,6 +145,7 @@ char *strstr(const char *, const char *);
 #define		PS_RETAINED	26	/* message retained (internal use) */
 #define		PS_REPOLL	28	/* repoll immediately with changed parameters (internal use) */
 #define		PS_IDLETIMEOUT	29	/* timeout on imap IDLE (internal use) */
+#define		PS_UNTAGGED	30	/* untagged response on imap command (internal use) */
 
 /* output noise level */
 #define         O_SILENT	0	/* mute, max squelch, etc. */
@@ -175,7 +176,7 @@ struct runctl
     char	*logfile;	/** where to write log information */
     char	*idfile;	/** where to store UID data */
     char	*pidfile;	/** where to record the PID of daemon mode processes */
-    char	*postmaster;
+    const char	*postmaster;
     char	*properties;
     int		poll_interval;	/** poll interval in seconds (daemon mode, 0 == off) */
     flag	bouncemail;
@@ -244,6 +245,8 @@ struct method		/* describe methods for protocol state machine */
     flag retry;			/* can getrange poll for new messages? */
 };
 
+enum badheader { BHREJECT = 0, BHACCEPT };
+
 struct hostdata		/* shared among all user connections to given server */
 {
     /* rc file data */
@@ -252,7 +255,7 @@ struct hostdata		/* shared among all user connections to given server */
     struct idlist *akalist;		/* server name first, then akas */
     struct idlist *localdomains;	/* list of pass-through domains */
     int protocol;			/* protocol type */
-    char *service;			/* service name */
+    const char *service;		/* service name */
     int interval;			/* # cycles to skip between polls */
     int authenticate;			/* authentication mode to try */
     int timeout;			/* inactivity timout in seconds */
@@ -269,6 +272,7 @@ struct hostdata		/* shared among all user connections to given server */
     flag tracepolls;			/* if TRUE, add poll trace info to Received */
     char *principal;			/* Kerberos principal for mail service */
     char *esmtp_name, *esmtp_password;	/* ESMTP AUTH information */
+    enum badheader badheader;		/* bad-header {pass|reject} */
 
 #if defined(linux) || defined(__FreeBSD__)
 #define CAN_MONITOR
@@ -320,7 +324,7 @@ struct query
     char *smtpaddress;		/* address to force in RCPT TO */ 
     char *smtpname;             /* full RCPT TO name, including domain */
     struct idlist *antispam;	/* list of listener's antispam response */
-    char *mda;			/* local MDA to pass mail to */
+    const char *mda;		/* local MDA to pass mail to */
     char *bsmtp;		/* BSMTP output file */
     char listener;		/* what's the listener's wire protocol? */
 #define SMTP_MODE	'S'
@@ -380,7 +384,7 @@ struct query
 
     /* internal use -- per-message state */
     int mimemsg;		/* bitmask indicating MIME body-type */
-    char digest[DIGESTLEN];	/* md5 digest buffer */
+    unsigned char digest[DIGESTLEN];	/* md5 digest buffer */
 
     /* internal use -- housekeeping */
     struct query *next;		/* next query control block in chain */
@@ -450,7 +454,7 @@ extern char *home;		/* home directory of invoking user */
 extern char *fmhome;		/* fetchmail home directory */
 extern int pass;		/* number of re-polling pass */
 extern flag configdump;		/* dump control blocks as Python dictionary */
-extern char *fetchmailhost;	/* either "localhost" or an FQDN */
+extern const char *fetchmailhost; /* either "localhost" or an FQDN */
 extern int suppress_tags;	/* suppress tags in tagged protocols? */
 extern char shroud[PASSWORDLEN*2+3];	/* string to shroud in debug output */
 #ifdef SDPS_ENABLE
@@ -555,6 +559,7 @@ extern volatile int lastsig;
 /* sink.c: forwarding */
 void smtp_close(struct query *, int);
 int smtp_open(struct query *);
+int smtp_setup(struct query *);
 char *rcpt_address(struct query *, const char *, int);
 int stuffline(struct query *, char *);
 int open_sink(struct query*, struct msgblk *, int*, int*);
@@ -633,10 +638,10 @@ int doETRN (struct query *);
 int doODMR (struct query *);
 
 /* authentication functions */
-int do_cram_md5(int sock, char *command, struct query *ctl, char *strip);
-int do_rfc1731(int sock, char *command, char *truename);
-int do_gssauth(int sock, char *command, char *service, char *hostname, char *username);
-int do_otp(int sock, char *command, struct query *ctl);
+int do_cram_md5(int sock, const char *command, struct query *ctl, const char *strip);
+int do_rfc1731(int sock, const char *command, const char *truename);
+int do_gssauth(int sock, const char *command, const char *service, const char *hostname, const char *username);
+int do_otp(int sock, const char *command, struct query *ctl);
 
 /* miscellanea */
 
@@ -646,8 +651,8 @@ extern char currentwd[1024], rcfiledir[1024];
 struct query *hostalloc(struct query *); 
 int parsecmdline (int, char **, struct runctl *, struct query *);
 char *prependdir (const char *, const char *);
-char *MD5Digest (unsigned const char *);
-void hmac_md5 (char *, size_t, char *, size_t, unsigned char *, size_t);
+unsigned char *MD5Digest (unsigned const char *);
+void hmac_md5 (const unsigned char *, size_t, const unsigned char *, size_t, unsigned char *, size_t);
 int POP3_auth_rpa(char *, char *, int socket);
 typedef RETSIGTYPE (*SIGHANDLERTYPE) (int);
 void deal_with_sigchld(void);
@@ -718,7 +723,7 @@ char *stpcpy(char *, const char*);
 #endif /* __CYGWIN__ */
 
 extern int mailserver_socket_temp;
-extern char *program_name;
+extern const char *program_name;
 
 /* POSIX space characters,
  * <tab>;<newline>;<vertical-tab>;<form-feed>;<carriage-return>;<space> */
