@@ -53,7 +53,8 @@ enum {
     LA_LIMITFLUSH,
     LA_IDLE,
     LA_NOSOFTBOUNCE,
-    LA_SOFTBOUNCE
+    LA_SOFTBOUNCE,
+    LA_BADHEADER
 };
 
 /* options still left: CgGhHjJoORTWxXYz */
@@ -93,6 +94,7 @@ static const struct option longoptions[] = {
   {"timeout",	required_argument, (int *) 0, 't' },
   {"envelope",	required_argument, (int *) 0, 'E' },
   {"qvirtual",	required_argument, (int *) 0, 'Q' },
+  {"bad-header",required_argument, (int *) 0, LA_BADHEADER},
 
   {"user",	required_argument, (int *) 0, 'u' },
   {"username",	required_argument, (int *) 0, 'u' },
@@ -283,7 +285,7 @@ int parsecmdline (int argc /** argument count */,
 	    rctl->logfile = prependdir (optarg, currentwd);
 	    break;
 	case LA_INVISIBLE:
-	    rctl->invisible = TRUE;
+	    rctl->invisible = FLAG_TRUE;
 	    break;
 	case LA_SHOWDOTS:
 	    rctl->showdots = FLAG_TRUE;
@@ -302,14 +304,25 @@ int parsecmdline (int argc /** argument count */,
 	    rctl->postmaster = (char *) xstrdup(optarg);
 	    break;
 	case LA_NOBOUNCE:
-	    run.bouncemail = FALSE;
+	    rctl->bouncemail = FLAG_FALSE;
 	    break;
 	case LA_NOSOFTBOUNCE:
-	    run.softbounce = FALSE;
+	    rctl->softbounce = FLAG_FALSE;
 	    break;
 	case LA_SOFTBOUNCE:
-	    run.softbounce = TRUE;
+	    rctl->softbounce = FLAG_TRUE;
 	    break;
+	case LA_BADHEADER:
+	    if (strcasecmp(optarg,"accept") == 0) {
+		ctl->server.badheader = BHACCEPT;
+	    } else if (strcasecmp(optarg,"reject") == 0) {
+		ctl->server.badheader = BHREJECT;
+	    } else {
+		fprintf(stderr,GT_("Invalid bad-header policy `%s' specified.\n"), optarg);
+		errflag++;
+	    }
+	    break;
+
 	case 'p':
 	    /* XXX -- should probably use a table lookup here */
 	    if (strcasecmp(optarg,"auto") == 0)
@@ -635,6 +648,8 @@ int parsecmdline (int argc /** argument count */,
 #endif
 	P(GT_("      --plugin      specify external command to open connection\n"));
 	P(GT_("      --plugout     specify external command to open smtp connection\n"));
+	P(GT_("      --bad-header {reject|accept}\n"
+	      "                    specify policy for handling messages with bad headers\n"));
 
 	P(GT_("  -p, --protocol    specify retrieval protocol (see man page)\n"));
 	P(GT_("  -U, --uidl        force the use of UIDLs (pop3 only)\n"));
