@@ -95,41 +95,44 @@
      SPAMBOUNCE = 311,
      SOFTBOUNCE = 312,
      SHOWDOTS = 313,
-     PROTO = 314,
-     AUTHTYPE = 315,
-     STRING = 316,
-     NUMBER = 317,
-     NO = 318,
-     KEEP = 319,
-     FLUSH = 320,
-     LIMITFLUSH = 321,
-     FETCHALL = 322,
-     REWRITE = 323,
-     FORCECR = 324,
-     STRIPCR = 325,
-     PASS8BITS = 326,
-     DROPSTATUS = 327,
-     DROPDELIVERED = 328,
-     DNS = 329,
-     SERVICE = 330,
-     PORT = 331,
-     UIDL = 332,
-     INTERVAL = 333,
-     MIMEDECODE = 334,
-     IDLE = 335,
-     CHECKALIAS = 336,
-     SSL = 337,
-     SSLKEY = 338,
-     SSLCERT = 339,
-     SSLPROTO = 340,
-     SSLCERTCK = 341,
-     SSLCERTPATH = 342,
-     SSLCOMMONNAME = 343,
-     SSLFINGERPRINT = 344,
-     PRINCIPAL = 345,
-     ESMTPNAME = 346,
-     ESMTPPASSWORD = 347,
-     TRACEPOLLS = 348
+     BADHEADER = 314,
+     ACCEPT = 315,
+     REJECT_ = 316,
+     PROTO = 317,
+     AUTHTYPE = 318,
+     STRING = 319,
+     NUMBER = 320,
+     NO = 321,
+     KEEP = 322,
+     FLUSH = 323,
+     LIMITFLUSH = 324,
+     FETCHALL = 325,
+     REWRITE = 326,
+     FORCECR = 327,
+     STRIPCR = 328,
+     PASS8BITS = 329,
+     DROPSTATUS = 330,
+     DROPDELIVERED = 331,
+     DNS = 332,
+     SERVICE = 333,
+     PORT = 334,
+     UIDL = 335,
+     INTERVAL = 336,
+     MIMEDECODE = 337,
+     IDLE = 338,
+     CHECKALIAS = 339,
+     SSL = 340,
+     SSLKEY = 341,
+     SSLCERT = 342,
+     SSLPROTO = 343,
+     SSLCERTCK = 344,
+     SSLCERTPATH = 345,
+     SSLCOMMONNAME = 346,
+     SSLFINGERPRINT = 347,
+     PRINCIPAL = 348,
+     ESMTPNAME = 349,
+     ESMTPPASSWORD = 350,
+     TRACEPOLLS = 351
    };
 #endif
 /* Tokens.  */
@@ -189,41 +192,44 @@
 #define SPAMBOUNCE 311
 #define SOFTBOUNCE 312
 #define SHOWDOTS 313
-#define PROTO 314
-#define AUTHTYPE 315
-#define STRING 316
-#define NUMBER 317
-#define NO 318
-#define KEEP 319
-#define FLUSH 320
-#define LIMITFLUSH 321
-#define FETCHALL 322
-#define REWRITE 323
-#define FORCECR 324
-#define STRIPCR 325
-#define PASS8BITS 326
-#define DROPSTATUS 327
-#define DROPDELIVERED 328
-#define DNS 329
-#define SERVICE 330
-#define PORT 331
-#define UIDL 332
-#define INTERVAL 333
-#define MIMEDECODE 334
-#define IDLE 335
-#define CHECKALIAS 336
-#define SSL 337
-#define SSLKEY 338
-#define SSLCERT 339
-#define SSLPROTO 340
-#define SSLCERTCK 341
-#define SSLCERTPATH 342
-#define SSLCOMMONNAME 343
-#define SSLFINGERPRINT 344
-#define PRINCIPAL 345
-#define ESMTPNAME 346
-#define ESMTPPASSWORD 347
-#define TRACEPOLLS 348
+#define BADHEADER 314
+#define ACCEPT 315
+#define REJECT_ 316
+#define PROTO 317
+#define AUTHTYPE 318
+#define STRING 319
+#define NUMBER 320
+#define NO 321
+#define KEEP 322
+#define FLUSH 323
+#define LIMITFLUSH 324
+#define FETCHALL 325
+#define REWRITE 326
+#define FORCECR 327
+#define STRIPCR 328
+#define PASS8BITS 329
+#define DROPSTATUS 330
+#define DROPDELIVERED 331
+#define DNS 332
+#define SERVICE 333
+#define PORT 334
+#define UIDL 335
+#define INTERVAL 336
+#define MIMEDECODE 337
+#define IDLE 338
+#define CHECKALIAS 339
+#define SSL 340
+#define SSLKEY 341
+#define SSLCERT 342
+#define SSLPROTO 343
+#define SSLCERTCK 344
+#define SSLCERTPATH 345
+#define SSLCOMMONNAME 346
+#define SSLFINGERPRINT 347
+#define PRINCIPAL 348
+#define ESMTPNAME 349
+#define ESMTPPASSWORD 350
+#define TRACEPOLLS 351
 
 
 
@@ -237,7 +243,7 @@ typedef union YYSTYPE
   char *sval;
 }
 /* Line 1489 of yacc.c.  */
-#line 241 "rcfile_y.h"
+#line 247 "rcfile_y.h"
 	YYSTYPE;
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
