@@ -75,20 +75,15 @@ static ssize_t cygwin_read(int sock, void *buf, size_t count);
 
 /* We need to define h_errno only if it is not already */
 #ifndef h_errno
-
-#ifdef HAVE_RES_SEARCH
-/* some versions of FreeBSD should declare this but don't */
+# if !HAVE_DECL_H_ERRNO
 extern int h_errno;
-#else
-/* pretend we have h_errno to avoid some #ifdef's later */
-static int h_errno;
-#endif
-
+# endif
 #endif /* ndef h_errno */
 
 #ifdef HAVE_SOCKETPAIR
 static char *const *parse_plugin(const char *plugin, const char *host, const char *service)
-{	const char **argvec;
+{
+	char **argvec;
 	const char *c, *p;
 	char *cp, *plugin_copy;
 	unsigned int plugin_copy_len;
@@ -135,25 +130,25 @@ static char *const *parse_plugin(const char *plugin, const char *host, const cha
 	}
 	plugin_copy[plugin_copy_len] = 0;
 
-	argvec = (const char **)malloc(s);
+	argvec = (char **)malloc(s);
 	if (!argvec)
 	{
 		report(stderr, GT_("fetchmail: malloc failed\n"));
 		return NULL;
 	}
 	memset(argvec, 0, s);
-	for (c = p = plugin_copy, i = 0; *c; c++)
-	{	if ((!isspace((unsigned char)*c)) && (c == p ? 1 : isspace((unsigned char)*p))) {
-			argvec[i] = c;
+	for (p = cp = plugin_copy, i = 0; *cp; cp++)
+	{	if ((!isspace((unsigned char)*cp)) && (cp == p ? 1 : isspace((unsigned char)*p))) {
+			argvec[i] = cp;
 			i++;
 		}
-		p = c;
+		p = cp;
 	}
 	for (cp = plugin_copy; *cp; cp++)
 	{	if (isspace((unsigned char)*cp))
 			*cp = 0;
 	}
-	return (char *const*)argvec;
+	return argvec;
 }
 
 static int handle_plugin(const char *host,
@@ -391,7 +386,7 @@ static	SSL *_ssl_context[FD_SETSIZE];
 static SSL	*SSLGetContext( int );
 #endif /* SSL_ENABLE */
 
-int SockWrite(int sock, char *buf, int len)
+int SockWrite(int sock, const char *buf, int len)
 {
     int n, wrlen = 0;
 #ifdef	SSL_ENABLE
@@ -462,7 +457,7 @@ int SockRead(int sock, char *buf, int len)
 			/* SSL_peek says no data...  Does he mean no data
 			or did the connection blow up?  If we got an error
 			then bail! */
-			if( 0 != ( n = SSL_get_error(ssl, n) ) ) {
+			if (0 != SSL_get_error(ssl, n)) {
 				return -1;
 			}
 			/* We didn't get an error so read at least one
@@ -542,7 +537,7 @@ int SockPeek(int sock)
 			/* SSL_peek says 0...  Does that mean no data
 			or did the connection blow up?  If we got an error
 			then bail! */
-			if( 0 != ( n = SSL_get_error(ssl, n) ) ) {
+			if(0 != SSL_get_error(ssl, n)) {
 				return -1;
 			}
 
@@ -628,9 +623,10 @@ static int SSL_verify_callback( int ok_return, X509_STORE_CTX *ctx, int strict )
 				report(stdout, GT_("Unknown Issuer CommonName\n"));
 		}
 		if ((i = X509_NAME_get_text_by_NID(subj, NID_commonName, buf, sizeof(buf))) != -1) {
-			if (outlevel >= O_VERBOSE)
+			if (outlevel >= O_VERBOSE) {
 				report(stdout, GT_("Server CommonName: %s\n"), (tt = sdump(buf, i)));
-			xfree(tt);
+				xfree(tt);
+			}
 			if ((size_t)i >= sizeof(buf) - 1) {
 				/* Possible truncation. In this case, this is a DNS name, so this
 				 * is really bad. We do not tolerate this even in the non-strict case. */
@@ -818,16 +814,16 @@ static const char *SSLCertGetCN(const char *mycert,
  * uses SSL *ssl global variable, which is currently defined
  * in this file
  */
-int SSLOpen(int sock, char *mycert, char *mykey, char *myproto, int certck, char *certpath,
+int SSLOpen(int sock, char *mycert, char *mykey, const char *myproto, int certck, char *certpath,
     char *fingerprint, char *servercname, char *label, char **remotename)
 {
         struct stat randstat;
         int i;
 
 	SSL_load_error_strings();
-	SSLeay_add_ssl_algorithms(); /* synonym for SSL_library_init() */
-	
-#ifdef SSL_ENABLE
+	SSL_library_init();
+	OpenSSL_add_all_algorithms(); /* see Debian Bug#576430 and manpage */
+
         if (stat("/dev/random", &randstat)  &&
             stat("/dev/urandom", &randstat)) {
           /* Neither /dev/random nor /dev/urandom are present, so add
@@ -843,8 +839,6 @@ int SSLOpen(int sock, char *mycert, char *mykey, char *myproto, int certck, char
             RAND_add (buf, sizeof buf, 0.1);
           }
         }
-#endif /* SSL_ENABLE */
-
 
 	if( sock < 0 || (unsigned)sock > FD_SETSIZE ) {
 		report(stderr, GT_("File descriptor out of range for SSL") );
