@@ -44,14 +44,21 @@ int SockPeek(int sock);
 Write a chunk of bytes to the socket (matches interface of fwrite).
 Returns number of bytes successfully written.
 */
-int SockWrite(int sock, char *buf, int size);
+int SockWrite(int sock, const char *buf, int size);
+
+/* from /usr/include/sys/cdefs.h */
+#if !defined __GNUC__ || __GNUC__ < 2
+# define __attribute__(xyz)    /* Ignore. */
+#endif
 
 /* 
 Send formatted output to the socket (matches interface of fprintf).
 Returns number of bytes successfully written.
 */
 #if defined(HAVE_STDARG_H)
-int SockPrintf(int sock, const char *format, ...) ;
+int SockPrintf(int sock, const char *format, ...)
+    __attribute__ ((format (printf, 2, 3)))
+    ;
 #else
 int SockPrintf();
 #endif
@@ -68,7 +75,7 @@ FIXME: document this
 int UnixOpen(const char *path);
 
 #ifdef SSL_ENABLE
-int SSLOpen(int sock, char *mycert, char *mykey, char *myproto, int certck, char *certpath,
+int SSLOpen(int sock, char *mycert, char *mykey, const char *myproto, int certck, char *certpath,
     char *fingerprint, char *servercname, char *label, char **remotename);
 #endif /* SSL_ENABLE */
 
